describe('TaskControllerTest', function() {
  var controller;
  var httpBackend;
  var document;

  var sampleData = [
    {name: 'Practice TDD', month: 3, day: 31, year: 2015},
    {name: 'Get an oz of Discipline', month: 3, day: 3, year: 2015},    
  ]
  
  beforeEach(module('todoApp'));
  
  beforeEach(inject(function($controller, _$httpBackend_) {
    controller = $controller('TasksController');
    httpBackend = _$httpBackend_
  }));
  
  it('canary test should pass', function() {
    expect(true).to.eql(true);
  });
  
  it('controller should have empty tasks', function() {    
    expect(controller.tasks).to.eql([]);
  });  
  
  it('getTasks should return tasks from the server', function() {    
    
    httpBackend.expectGET('tasks')
               .respond(200, sampleData);
    controller.getTasks();
    httpBackend.flush();
    
    expect(controller.tasks).to.include(sampleData[0]);
    expect(controller.tasks).to.include(sampleData[1]);
  });
  
  it('tasks should be sorted by ascending order of year', function() {
    var tasks = [
      {name: 'Practice TDD', month: 3, day: 31, year: 2015},
      {name: 'Get an oz of Discipline', month: 3, day: 3, year: 2014},    
    ]
    var sortedTasks = controller.sortTasks(tasks);
    expect(sortedTasks[0]).to.eql(tasks[1]);
  });
  
  it('tasks should be sorted by ascending order of month if years match', function() {
    var tasks = [
      {name: 'Refactor', month: 3, day: 31, year: 2015},
      {name: 'Practice TDD', month: 1, day: 31, year: 2015},
      {name: 'Get an oz of Discipline', month: 3, day: 3, year: 2015},    
    ]
    var sortedTasks = controller.sortTasks(tasks);
    expect(sortedTasks[0]).to.eql(tasks[1]);
  });

  it('tasks should be sorted by ascending order of days if years and month match', function() {
    var tasks = [
      {name: 'Refactor', month: 1, day: 3, year: 2015},
      {name: 'Practice TDD', month: 1, day: 31, year: 2015},
      {name: 'Get an oz of Discipline', month: 1, day: 1, year: 2015},    
    ]
    var sortedTasks = controller.sortTasks(tasks);
    expect(sortedTasks[0]).to.eql(tasks[2]);
  });

  it('tasks should be sorted by ascending order of name if other date fields match', function() {
    var tasks = [
      {name: 'Refactor', month: 1, day: 3, year: 2015},
      {name: 'Practice TDD', month: 1, day: 3, year: 2015},
      {name: 'Get an oz of Discipline', month: 1, day: 3, year: 2015},    
    ]
    var sortedTasks = controller.sortTasks(tasks);
    expect(sortedTasks[0]).to.eql(tasks[2]);
    expect(sortedTasks[1]).to.eql(tasks[1]);
  });
  
  it('getTasks calls sortTasks', function() {    

    var givenData = [];
    controller.sortTasks = function(data) { givenData = data; }
    
    httpBackend.expectGET('tasks')
               .respond(200, sampleData);
    controller.getTasks();
    httpBackend.flush();
    
    expect(givenData).to.eql(sampleData);
  });

  describe("DocumentSetTest", function() {
    var givenFunction;

    beforeEach(inject(function($controller, _$document_) {
      var document = _$document_;
      document.ready = function(fn) { givenFunction = fn; }
      
      controller = $controller('TasksController');
    }));
    
    it('getTask should be called on document ready', function() {    
      expect(givenFunction).to.eql(controller.getTasks);
    });
  });
});